@@ -6,7 +6,7 @@
 # .kind	      : python source
 # .author     : Marco Abrate
 # .site	      : Torino - Italy
-# .creation   :	23-Nov-2011
+# .creation   :	10-Mar-2012
 # .copyright  :	(c) 2011 Marco Abrate
 # .license    : GNU General Public License
 #
@@ -30,13 +30,14 @@
 
 import math, cmath
 import numpy as np
+import scipy
 
 ## module information
 ## qmath - a Python module to manipulate quaternions
 
 __author__ = 'Marco Abrate <abrate.m@gmail.com>'
 __license__ = '>= GPL v3'
-__version__ = '0.2.0'
+__version__ = '0.3.0'
 
 
 class AlgebraicError(Exception):
@@ -49,7 +50,7 @@ class AlgebraicError(Exception):
 class quaternion:
     "Quaternion algebra"
     
-    def __init__(self, q, vector = None):
+    def __init__(self, q, vector = None, matrix = np.identity(3)):
         """
         Initializes the quaternion.
         The following types can be converted to quaternion:
@@ -75,7 +76,7 @@ class quaternion:
         >>> q.__class__ == quaternion
         True
         """
-        
+        self.matrix = matrix
         if q.__class__ == quaternion:
             self.q = q.q
 
@@ -114,6 +115,45 @@ class quaternion:
     
         else:
             pass
+          
+        if matrix.shape == (3,3) and abs(matrix - matrix.transpose()).sum() == 0:
+
+            if (matrix - np.identity(3)).__abs__().sum() == 0:
+                self.prod_tensor = np.array([[[1, 0, 0, 0],
+                                              [0,-1, 0, 0],
+                                              [0, 0,-1, 0],
+                                              [0, 0, 0,-1]],
+                                             [[0, 1, 0, 0],
+                                              [1, 0, 0, 0],
+                                              [0, 0, 0, 1],
+                                              [0, 0,-1, 0]],
+                                             [[0, 0, 1, 0],
+                                              [0, 0, 0,-1],
+                                              [1, 0, 0, 0],
+                                              [0, 1, 0, 0]],
+                                             [[0, 0, 0, 1],
+                                              [0, 0, 1, 0],
+                                              [0,-1, 0, 0],
+                                              [1, 0, 0, 0]]])
+            else:
+                self.prod_tensor = np.array([[[1, 0, 0, 0],
+                                              [0,-(matrix[1,1]*matrix[2,2]-matrix[1,2]*matrix[2,1]), (matrix[1,0]*matrix[2,2]-matrix[1,2]*matrix[2,0]),-(matrix[1,0]*matrix[2,1]-matrix[1,1]*matrix[2,0])],
+                                              [0, (matrix[1,0]*matrix[2,2]-matrix[1,2]*matrix[2,0]),-(matrix[0,0]*matrix[2,2]-matrix[0,2]*matrix[2,0]), (matrix[0,0]*matrix[2,1]-matrix[0,1]*matrix[2,0])],
+                                              [0,-(matrix[1,0]*matrix[2,1]-matrix[1,1]*matrix[2,0]), (matrix[0,0]*matrix[2,1]-matrix[0,1]*matrix[2,0]),-(matrix[0,0]*matrix[1,1]-matrix[0,1]*matrix[1,0])]],
+                                             [[0, 1, 0, 0],
+                                              [1, 0, matrix[0,2],-matrix[0,1]],
+                                              [0,-matrix[0,2], 0, matrix[0,0]],
+                                              [0,matrix[0,1],-matrix[0,0], 0]],
+                                             [[0, 0, 1, 0],
+                                              [0, 0, matrix[1,2],-matrix[1,1]],
+                                              [1, -matrix[1,2], 0, matrix[0,1]],
+                                              [0, matrix[1,1], -matrix[0,1], 0]],
+                                             [[0, 0, 0, 1],
+                                              [0, 0, matrix[2,2], -matrix[1,2]],
+                                              [0,-matrix[2,2], 0, matrix[0,2]],
+                                              [1, matrix[1,2], -matrix[0,2], 0]]])
+        else:
+            raise AlgebraicError, 'matrix must be symmetric 3X3'
         return
 
     
@@ -381,14 +421,24 @@ class quaternion:
         >>> a *= b
         >>> a
         (11+2i-7j+24k)
+        >>> a = qmathcore.quaternion([0,0,0,1], matrix = np.array([[-1,0,0],[0,1,0],[0,0,-1]]))
+        >>> b = qmathcore.quaternion([0,0,0,1], matrix = np.array([[-1,0,0],[0,1,0],[0,0,-1]]))
+        >>> a *= b
+        >>> a
+        (1.0)
+        >>> a = qmathcore.quaternion([0,-1,0,-1], matrix = np.array([[-1,0,0],[0,1,0],[0,0,-1]]))
+        >>> b = qmathcore.quaternion([0,0,2,0], matrix = np.array([[-1,0,0],[0,1,0],[0,0,-1]]))
+        >>> a *= b
+        >>> a
+        (-2.0i+2.0k)
         """
         self.p = quaternion(other)
         try:
-            self.vect = self.q[0] * self.p.q[1:] + self.p.q[0] * self.q[1:] + np.cross(self.q[1:],self.p.q[1:])
-            return self.__class__(np.array([self.q[0] * self.p.q[0] - np.dot(self.q[1:],self.p.q[1:])] + self.vect.tolist()))
+            self.vect = np.dot(np.dot(np.array([self.q[0],self.q[1],self.q[2],self.q[3]]),self.prod_tensor),np.array([self.p.q[0],self.p.q[1],self.p.q[2],self.p.q[3]]))
+            return self.__class__(self.vect, matrix = self.matrix)
             
         except:
-            return quaternion(other)
+            return self.__class__(other,matrix=self.matrix)
 
     def __idiv__(self, other):
         """
@@ -486,6 +536,14 @@ class quaternion:
         >>> b = qmathcore.hurwitz(3-4j)
         >>> a * b
         (11+2i-7j+24k)
+        >>> a = qmathcore.quaternion([0,0,0,1], matrix = np.array([[-1,0,0],[0,1,0],[0,0,-1]]))
+        >>> b = qmathcore.quaternion([0,0,0,1], matrix = np.array([[-1,0,0],[0,1,0],[0,0,-1]]))
+        >>> a * b
+        (1.0)
+        >>> a = qmathcore.quaternion([0,-1,0,-1], matrix = np.array([[-1,0,0],[0,1,0],[0,0,-1]]))
+        >>> b = qmathcore.quaternion([0,0,2,0], matrix = np.array([[-1,0,0],[0,1,0],[0,0,-1]]))
+        >>> a * b
+        (-2.0i+2.0k)
         """
         self *= other
         return self
@@ -588,6 +646,11 @@ class quaternion:
         (-1+2i+2k)
         >>> pow(qmathcore.hurwitz('1+1i+1k'),2,3)
         (2+2i+2k)
+        >>> a = qmathcore.quaternion([0,-1,0,-1], matrix = np.array([[-1,0,0],[0,1,0],[0,0,-1]]))
+        >>> a ** 3
+        (2.0i+2.0k)
+        >>> a ** 4
+        (4.0)
         """
         if self.__class__ == quaternion:
             if int(exponent) == exponent:
@@ -735,7 +798,7 @@ class quaternion:
         >>> a.norm()
         30
         """
-        return self.q[0] ** 2 + self.q[1] ** 2 + self.q[2] ** 2 + self.q[3] ** 2
+        return np.dot(np.dot(np.array([self.q[0],self.q[1],self.q[2],self.q[3]]),self.prod_tensor[0]),np.array([self.q[0],-self.q[1],-self.q[2],-self.q[3]]))
 
     def delta(self):
         """
@@ -748,6 +811,7 @@ class quaternion:
         -29
         """
         return self.q[0] ** 2 - self.norm()
+        
 
     def inverse(self,modulo = None):
         """
@@ -859,7 +923,7 @@ class quaternion:
 class hurwitz(quaternion):
     "Quaternion algebra over integers"
     
-    def __init__(self, att):
+    def __init__(self, att, matrix=np.identity(3)):
         """
         Initializes the quaternion.
         The following types can be converted to quaternion:
@@ -882,7 +946,9 @@ class hurwitz(quaternion):
         >>> qmathcore.quaternion(q)
         (2.0j)
         """
-        self.q = quaternion(att)._int_()
+        self.matrix = matrix
+        self.q = quaternion(att, matrix = matrix)._int_()
+        self.prod_tensor = self.q.prod_tensor
 
 
         
